#include <iostream>
#include <cstring>
#include <cmath>

using namespace std;

class Businessman
{
private:
    char *name;
    int year;
    double wage;
    void print()
    {
        cout << "Ime biznismena je: " << name << ", godina rodjenja je: " << year << ", dok mu je plata: " << wage << endl;
    }
public:
    Businessman() : name(0) {};
    Businessman(char *, int, double);
    Businessman(const Businessman &);
    ~Businessman()
    {
        delete [] name;
        name = 0;
    }
    void setName(char * newName)
    {
        delete [] name;
        name = new char[strlen(newName) + 1];
        strcpy(name, newName);
    }
    void setYear(int newYear)
    {
        year = newYear;
    }
    void setWage(double newWage)
    {
        wage = newWage;
        if (wage > topWage) updateStaticFields(name, wage);
    }
    char * getName()  const
    {
        return name;
    }
    int getYear()  const
    {
        return year;
    }
    double getWage() const
    {
        return wage;
    }
    Businessman & operator=(const Businessman &);
    Businessman operator-(const Businessman &);
    Businessman & operator++();
    Businessman operator++(int);
    friend Businessman & richer(Businessman &, Businessman &);
    static char *topName;
    static double topWage;
    static void updateStaticFields(char *, double);
    friend class Chamber;
};

char * Businessman::topName = 0;
double Businessman::topWage = 0.0;

void Businessman::updateStaticFields(char *newName, double newWage)
{
    delete [] topName;
    topName = new char[strlen(newName) + 1];
    strcpy(topName, newName);
    topWage = newWage;
}

Businessman::Businessman(char *_name, int _year, double _wage) : year(_year), wage(_wage)
{
    name = new char[strlen(_name) + 1];
    strcpy(name, _name);
    if (wage > topWage) updateStaticFields(name, wage);
}

Businessman::Businessman(const Businessman & businessman) : year(businessman.year), wage(businessman.wage)
{
    name = new char[strlen(businessman.name) + 1];
    strcpy(name, businessman.name);
}

Businessman & Businessman::operator=(const Businessman & businessman)
{
    if (this != &businessman)
    {
        delete [] name;
        name = new char[strlen(businessman.name) + 1];
        strcpy(name, businessman.name);
        year = businessman.year;
        wage = businessman.wage;
    }
    return *this;
}

Businessman Businessman::operator-(const Businessman &businessman)
{
    if (wage < businessman.wage) return Businessman(name, year, fabs(wage - businessman.wage));
    return Businessman(businessman.name, businessman.year, fabs(wage - businessman.wage));
}

Businessman & Businessman::operator++()
{
    wage += 100000;
    return *this;
}

Businessman Businessman::operator++(int k)
{
    Businessman temp(*this);
    wage += 100000;
    return temp;
}

Businessman & richer(Businessman & b1, Businessman &b2)
{
    double totalFirst = (2022 - b1.year) * b1.wage;
    double totalSecond = (2022 - b2.year) * b2.wage;
    if (totalFirst > totalSecond) return b1;
    return b2;
}

class Chamber
{
private:
    Businessman *arr;
    int len;
public:
    Chamber() : arr(0) {};
    Chamber(Businessman *, int);
    Chamber(const Chamber &);
    Chamber & operator=(const Chamber &);
    ~Chamber()
    {
        delete [] arr;
        arr = 0;
    }
    void setArr(Businessman *, int);
    Businessman * getArr() const
    {
        return arr;
    }
    int getLen() const
    {
        return len;
    }
    Businessman operator[](int);
    Businessman president();
    void print();
};

Chamber::Chamber(Businessman *_arr, int _len) : len(_len)
{
    arr = new Businessman[len];
    for (int i = 0; i<len; i++)
    {
        arr[i] = _arr[i];
    }
}

Chamber & Chamber::operator=(const Chamber & chamber)
{
    if(this != &chamber)
    {
        len = chamber.len;
        delete [] arr;
        arr = new Businessman[len];
        for (int i = 0; i<len; i++)
        {
            arr[i] = chamber.arr[i];
        }
    }
    return *this;
}

Businessman Chamber::operator[](int ind)
{
    if (ind >= 0 && ind <= len - 1) return arr[ind];
    return Businessman("Biznis Kontroverz", 1991, 5000000.0);

}

Businessman Chamber::president()
{
    Businessman prez = arr[0];
    for (int i=1; i< len; i++)
        if (arr[i].year < prez.year)
            prez = arr[i];
    return prez;
}

void Chamber::setArr(Businessman *newArr, int newLen)
{
    len = newLen;
    delete [] arr;
    arr = new Businessman[len];
    for (int i = 0; i<len; i++)
    {
        arr[i] = newArr[i];
    }
}

void Chamber::print()
{
    for(int i=0; i<len; i++)
        arr[i].print();
}

int main()
{
    char name[50];
    int year;
    double wage;
    cout << "Unesite ime, godinu i platu:" << endl;
    cin >> name >> year >> wage;
    Businessman bizz1(name, year, wage);
    cout << "Unesite ime, godinu i platu:" << endl;
    cin >> name >> year >> wage;
    Businessman bizz2(name, year, wage);
    Businessman bizz3 = bizz2 - bizz1;
    bizz3++;
    ++bizz3;
    bizz3 = richer(bizz1, bizz2);
    cout << "Top plata je: " << Businessman::topWage << ", i ima je: " << Businessman::topName << endl;
    cout << "Koliko clanova ima vasa komora: ";
    int len;
    cin >> len;
    Businessman arr[len];
    for (int i=0; i<len; i++)
    {
        cout << "Unesite ime, godinu i platu:" << endl;
        cin >> name >> year >> wage;
        arr[i] = Businessman(name, year, wage);
    }
    Chamber komora(arr, len);
    Businessman b1, b2, prez;
    b1 = komora[303];
    b2 = komora[0];
    prez = komora.president();

}
